package de.cscherr.mcpht.config;

import de.cscherr.mcpht.MCPHTClient;
import de.cscherr.mcpht.mixin.client.NetworkPacketLogMixin;
import me.shedaniel.autoconfig.ConfigData;
import me.shedaniel.autoconfig.annotation.Config;
import me.shedaniel.autoconfig.annotation.ConfigEntry;
import net.minecraft.block.Block;
import net.minecraft.block.RedstoneBlock;

import java.util.ArrayList;
import java.util.regex.Pattern;

@Config(name = "mcphtconf")
public class MCPHTConfig implements ConfigData {
    public void onSave() {
        MCPHTClient.LOGGER.info("saving and processing configs");
        MCPHTClient.networkPacketFilter = compileFilter();

        logSettings();
    }
    public void onLoad() {
        MCPHTClient.LOGGER.info("loading configs");
        MCPHTClient.networkPacketFilter = compileFilter();

        logSettings();
    }
    public void logSettings() {
        MCPHTClient.LOGGER.info(String.format("CONFIG: \n" +
                "NetworkLogging\n" +
                "	RX: %b\n" +
                "	TX: %b\n" +
                "	verbosity: %s\n" +
                "	regex: '%s'\n" +
                "	filter: %s\n" +
                "Rendering\n" +
                "   enabled: %b",
                MCPHTClient.CONFIG.networkLogging.RX,
                MCPHTClient.CONFIG.networkLogging.TX,
                MCPHTClient.CONFIG.networkLogging.verbosity,
                MCPHTClient.CONFIG.networkLogging.regex,
                MCPHTClient.networkPacketFilter,

                MCPHTClient.CONFIG.xRay.enable
        ));
    }
    private Pattern compileFilter() {
        try {
            return Pattern.compile(this.networkLogging.regex);
        } catch (Exception e) {
            MCPHTClient.LOGGER.warn(String.format("Invalid regex: %s", this.networkLogging.regex));
            return Pattern.compile(".*"); // just accept all on error
        }
    }
    public static class NetworkLoggingScheme {
        public boolean RX = false;
        public boolean TX = false;
        public NetworkPacketVerbosity verbosity = NetworkPacketVerbosity.NAME;
        public String regex = ".*";
        public enum NetworkPacketVerbosity {
            ALL, NAME
        }
    }
    @ConfigEntry.Category("Networking")
    @ConfigEntry.Gui.CollapsibleObject
    public NetworkLoggingScheme networkLogging = new NetworkLoggingScheme();
    @ConfigEntry.Category("Rendering")
    @ConfigEntry.Gui.CollapsibleObject
    public XRayScheme xRay = new XRayScheme();
    // TODO: toggle with hotkey
    public static class XRayScheme {
        public boolean enable = false;
    }
}
